/****** PIDbin.CPP ******
Copyright (c) 2002 by
Edwin A. Suominen
Author of the PRIVARIA Secure Networking Suite
OSI-Approved Open-Source Software available at www.privaria.org

Permission is hereby granted, free of charge, to any person obtaining a copy of this
software and associated documentation files (the "Software"), to deal in the Software
without restriction, including without limitation the rights to use, copy, modify,
merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to the following
conditions:

The above copyright notice and this permission notice shall be included in all copies
or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
******************/
#include <windows.h>
#include <tlhelp32.h>
#include <string>
#include <sstream>

/*** HEADERS ***/
// processes    : lists current processes by PID
void processes(std::string & str);
// kill         : kills process by PID
void killp(unsigned pid);


/*** SWIG'd FUNCTIONS ***/
void processes(std::string & str)
{
    // String & Stream Stuff
    std::ostringstream strout;
    
    // (FROM Q175030) ToolHelp Function Pointers.
    HANDLE (WINAPI *lpfCreateToolhelp32Snapshot)(DWORD, DWORD);
    BOOL (WINAPI *lpfProcess32First)(HANDLE, LPPROCESSENTRY32);
    BOOL (WINAPI *lpfProcess32Next)(HANDLE, LPPROCESSENTRY32);

    // (FROM Q175030) Other declarations
    HINSTANCE      hInstLib  = NULL;
    HINSTANCE      hInstLib2 = NULL;
    HANDLE         hSnapShot = NULL;
    LPDWORD        lpdwPIDs  = NULL;
    PROCESSENTRY32 procentry;
    BOOL           bFlag;

    hInstLib = LoadLibraryA("Kernel32.DLL");
    if (hInstLib == NULL)
        return;

    // (FROM Q175030) Get procedure addresses.
    lpfCreateToolhelp32Snapshot =
        (HANDLE (WINAPI *)(DWORD,DWORD))
        GetProcAddress(hInstLib, "CreateToolhelp32Snapshot");

    lpfProcess32First =
        (BOOL (WINAPI *)(HANDLE,LPPROCESSENTRY32))
        GetProcAddress(hInstLib, "Process32First");

    lpfProcess32Next =
        (BOOL (WINAPI *)(HANDLE,LPPROCESSENTRY32))
        GetProcAddress(hInstLib, "Process32Next");

    if (lpfProcess32Next == NULL
        || lpfProcess32First == NULL
        || lpfCreateToolhelp32Snapshot == NULL)
        return;

    // (FROM Q175030) Get a handle to a Toolhelp snapshot of all processes.
    hSnapShot = lpfCreateToolhelp32Snapshot(TH32CS_SNAPPROCESS, 0);
    if (hSnapShot == INVALID_HANDLE_VALUE) {
        FreeLibrary(hInstLib);
        return;
    }

    // (ADAPTED FROM Q175030) Get the first process' information.
    procentry.dwSize = sizeof(PROCESSENTRY32);
    bFlag = lpfProcess32First(hSnapShot, &procentry);
    if(bFlag == FALSE) {
        CloseHandle(hSnapShot);
        return;
    }

    // Loop until we've gotten all processes
    while(bFlag) {
        // Add a new line of text for this process entry
        strout << procentry.szExeFile << " ";
        strout << procentry.th32ProcessID << std::endl;

        // Get next process
        procentry.dwSize = sizeof(PROCESSENTRY32);
        bFlag = lpfProcess32Next(hSnapShot, &procentry);
    }

    // Close the snapshot handle
    CloseHandle(hSnapShot);

    // Transfer the collected streams to our string
    str = strout.str();

    return;
}


void killp(unsigned pid) {

	HANDLE process;

	process = OpenProcess(PROCESS_TERMINATE, 0, pid);
	TerminateProcess(process, (unsigned)-1);
	return;
}
