/****** PID.I ******
Copyright (c) 2002 by
Edwin A. Suominen
Author of the PRIVARIA Secure Networking Suite
OSI-Approved Open-Source Software available at www.privaria.org

Permission is hereby granted, free of charge, to any person obtaining a copy of this
software and associated documentation files (the "Software"), to deal in the Software
without restriction, including without limitation the rights to use, copy, modify,
merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to the following
conditions:

The above copyright notice and this permission notice shall be included in all copies
or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
******************/

%module PIDbin
%{
#include <string>
%}

/*** TYPEMAPS ***/
// The inputs are pointers to a string object for the string result
%typemap(tcl8,ignore) (std::string & str) {
    /* Create a new string object and pass on the reference to it */
    std::string sw;
    $1 = &sw;
}

// The outputs are the pointer and length of the completed string
%typemap(tcl8,argout) (std::string & str) {
    /* First extract the string result from its object to
       a null-terminated local character array */
    unsigned len = $1->length();
    char *ps = new char[len+1];
    $1->copy(ps,len);
    ps[len] = '\0';     // Must null-terminate string
    
    /* Now put the string into a TCL object as our return value,
       IF result object isn't shared (because of threads?) */
    if ( !Tcl_IsShared($result) ) {
        Tcl_Obj *o;
        o = Tcl_NewStringObj(ps,len);
        Tcl_IncrRefCount(o);

        Tcl_ListObjAppendElement(interp,$result,o);

        Tcl_DecrRefCount(o);

        /* Now delete our dynamically-allocated character array */
        delete [] ps;
        //delete $result;     // Do we need to do this?
        delete $1;
    }
}   


/* The function(s) we're wrapping */
extern void processes(std::string & str);
extern void killp(unsigned pid);


